/*
 *	DATA.C
 *	Tom Kerrigan's Simple Chess Program (TSCP)
 *
 *	Copyright 1997 Tom Kerrigan
 */


#include "defs.h"


/* the board representation */
int color[64];  /* LIGHT, DARK, or EMPTY */
int piece[64];  /* PAWN, KNIGHT, SILVER, LANCE, GOLD, KING, or EMPTY */
int side;  /* the side to move */
int xside;  /* the side not to move */
int last_capture;  /* the number of moves since a capture, to handle repetitions*/
int ply;  /* the half-move that we're on */

/* the number of pieces each player has got */
int pieces_count[2] = { 16, 16 };

/* gen_dat is some memory for move lists that are created by the move
   generators. The move list for ply n starts at gen_begin[n] of gen_dat.
   gen_end[n] is right after the move list. */
gen_t gen_dat[MOVE_STACK];
int gen_begin[HIST_STACK], gen_end[HIST_STACK];

/* the history heuristic array (used for move ordering) */
int history[64][64];

/* we need an array of hist_t's so we can take back the
   moves we make */
hist_t hist_dat[HIST_STACK];

/* the engine will search for max_time milliseconds or until it finishes
   searching max_depth ply. */
long max_time;
int max_depth;

/* the time when the engine starts searching, and when it should stop */
long start_time;
long stop_time;

long nodes;  /* the number of nodes we've searched */

/* a "triangular" PV array */
move pv[HIST_STACK][HIST_STACK];
int pv_length[HIST_STACK];
BOOL follow_pv;

/* undo_dat is some memory to save the moves from the game
   (not just from analysis) so the user can take back moves. */
hist_t undo_dat[UNDO_STACK];
int undos;


/* Now we have the mailbox array, so called because it looks like a
   mailbox, at least according to Bob Hyatt. This is useful when we
   need to figure out what pieces can go where. Let's say we have a
   rook on square a4 (32) and we want to know if it can move one
   square to the left. We subtract 1, and we get 31 (h5). The rook
   obviously can't move to h5, but we don't know that without doing
   a lot of annoying work. Sooooo, what we do is figure out a4's
   mailbox number, which is 61. Then we subtract 1 from 61 (60) and
   see what mailbox[60] is. In this case, it's -1, so it's out of
   bounds and we can forget it. You can see how mailbox[] is used
   in attack() in board.c. */

int mailbox[120] = {
	 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	 -1,  0,  1,  2,  3,  4,  5,  6,  7, -1,
	 -1,  8,  9, 10, 11, 12, 13, 14, 15, -1,
	 -1, 16, 17, 18, 19, 20, 21, 22, 23, -1,
	 -1, 24, 25, 26, 27, 28, 29, 30, 31, -1,
	 -1, 32, 33, 34, 35, 36, 37, 38, 39, -1,
	 -1, 40, 41, 42, 43, 44, 45, 46, 47, -1,
	 -1, 48, 49, 50, 51, 52, 53, 54, 55, -1,
	 -1, 56, 57, 58, 59, 60, 61, 62, 63, -1,
	 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1
};

int mailbox64[64] = {
	21, 22, 23, 24, 25, 26, 27, 28,
	31, 32, 33, 34, 35, 36, 37, 38,
	41, 42, 43, 44, 45, 46, 47, 48,
	51, 52, 53, 54, 55, 56, 57, 58,
	61, 62, 63, 64, 65, 66, 67, 68,
	71, 72, 73, 74, 75, 76, 77, 78,
	81, 82, 83, 84, 85, 86, 87, 88,
	91, 92, 93, 94, 95, 96, 97, 98
};


/* slide, offsets, and offset are basically the vectors that
   pieces can move in. If slide for the piece is FALSE, it can
   only move one square in any one direction. offsets is the
   number of directions it can move in, and offset is an array
   of the actual directions. */

BOOL slide[6] = {
  /* Pawn, Knight, Silver, Lance, Gold, King */
	FALSE, FALSE, FALSE, TRUE, FALSE, FALSE
};

int offsets[6] = {
  /* Pawn, Knight, Silver, Lance, Gold, King */
	1, 2, 5, 1, 6, 8
};

int promotion[6] = {
  GOLD, GOLD, GOLD, GOLD, 0, 0
};

int offset[2][6][8] = {
  /* sente's pieces */
{  { -10, 0, 0, 0, 0, 0, 0, 0 },       /* Pawn */
  { -21, -19, 0, 0, 0, 0, 0, 0 },     /* Knight */
  { -11, -9, 9, 11, -10, 0, 0, 0 },   /* Silver */
  { -10, 0, 0, 0, 0, 0, 0, 0 },       /* Lance */
  { -11, -10, -9, -1, 1, 10, 0,0 },  /* Gold */
  { -11, -10, -9, -1, 1, 9, 10, 11 }  /* King */
 }, /* gote's pieces */
 { {  10, 0, 0, 0, 0, 0, 0, 0 },       /* Pawn */
  {  21, 19, 0, 0, 0, 0, 0, 0 },     /* Knight */
  { -11, -9, 9, 11, 10, 0, 0, 0 },   /* Silver */
  {  10, 0, 0, 0, 0, 0, 0, 0 },       /* Lance */
  {  11, 10, 9, 1, -1, -10, 0, 0 },  /* Gold */
  { -11, -10, -9, -1, 1, 9, 10, 11 }  /* King */
} };

int can_promote_zone[2][6] = {{
  /* Pawn, Knight, Silver, Lance, Gold, King */
      24, 24, 24, 24, 0, 0 }, { 39, 39, 39, 39, 0, 0 }};

int must_promote_zone[2][6] = { {
  /* Pawn, Knight, Silver, Lance, Gold, King */
  8, 16, 0, 8, 0, 0 }, { 55, 47, 0, 55, 0, 0 }};

/* the piece letters, for print_board() */
char piece_char[6] = {
	'P', 'N', 'S', 'L', 'G', 'K'
};


/* the initial board state */

int init_color[64] = {
	1, 1, 1, 1, 1, 1, 1, 1,
	6, 6, 6, 6, 6, 6, 6, 6,
	1, 1, 1, 1, 1, 1, 1, 1,
	6, 6, 6, 6, 6, 6, 6, 6,
	6, 6, 6, 6, 6, 6, 6, 6,
	0, 0, 0, 0, 0, 0, 0, 0,
	6, 6, 6, 6, 6, 6, 6, 6,
	0, 0, 0, 0, 0, 0, 0, 0
};

int init_piece[64] = {
	3, 1, 2, 4, 5, 2, 1, 3,
	6, 6, 6, 6, 6, 6, 6, 6,
	0, 0, 0, 0, 0, 0, 0, 0,
	6, 6, 6, 6, 6, 6, 6, 6,
	6, 6, 6, 6, 6, 6, 6, 6,
	0, 0, 0, 0, 0, 0, 0, 0,
	6, 6, 6, 6, 6, 6, 6, 6,
	3, 1, 2, 5, 4, 2, 1, 3
};
